## 项目启动（两种方式）
- 走mock数据方式
前端启动伺服器：
npm run dev => 静态走dev-server(:8999) => /ajax/** 代理到node中台(:8888)

中台mock伺服器：
npm run start => 路由到/app/mock不进行登录校验和权限
最终页面访问：localhost:8999/

- 走测试数据库方式（推荐）
中台启动伺服器：
npm run dev_test(test) => 静态到/dist/目录 => 业务路由走/app/router
前端打包静态文件到/dist/目录：
npm run watch
最终页面访问：localhost:8888/


## 中台说明
+ 前端请求路由层分流
+ controller, 将前端请求路由到处理ctrl
+ service层, ctrl请求后端数据库(axios封装)
> 需要说明的是：
前端请求主要是: GET / POST
而后端是restful风格接口: GET / POST / PUT / DELETE
中间层需要转换映射转换

## 流程说明
1. 中台：校验是否登录（即校验是否有session，中间层`check/checkLogin`）,如果登录，跳步骤3；
2. 中台：未登录，则跳转通行证进行登录（controller层`login/goLogin`），登录完成，调用回调函数（controller层`login/callback`）,回调函数执行步骤：
- 调用通行证接口，获取token（service层`login/getToken`）;
- 调用通行证接口，获取用户信息（service层`login/getUserInfo`）;
- 调用运营平台接口，获取用户权限（service层`komp/permission/list`）,然后用户uid打上时间戳设置Cookie，同时将用户信息和权限信息设置redis缓存(controller层`login/getPermission`)

3. 进入前端首页，前端调用中台接口获取用户信息(controller层`user/userInfo`)、用户权限(controller层`inner/permission`的list),**所有带/ajax前缀的接口需要校验是否登入**，中台获取请求的cookies并校验：（备注：**所有带/ajax前缀的接口需要cookies校验是否登入**，）

- 校验通过：用户信息接口，中台从redis读取用户信息并返回；用户权限接口，中台从redis读取用户权限并返回；

- 校验不通过：未登入或者则登入信息超时则中台返回错误码11006，前端调用中台登出接口（`${CONFIG.host_url}/logout`），中台跳步骤5。

4. 前端获取用户权限时，如果有云产品管理权限则接着调用中台接口获取云产品管理列表（备注：**所有带/ajax/purview前缀的接口需要校验用户权限**），中台获取请求的cookies，用cookies中的uid从redis读取该用户权限，判断当前用户是否有这个接口的权限：

- 校验通过: 转发该请求到运营后端接口并将接口返回数据返回给前端。

- 校验不通过：中台返回错误码11006，前端调用中台登出接口（`${CONFIG.host_url}/logout`），中台跳步骤5。

5. 登出时，清除Cookie和redis（controller层`login/logout`），跳步骤2。

6. 页面其他大部分接口都是带`/ajax/purview`前缀，一般流程为：前端调用中台接口，中台获取请求的cookies，用cookies中的uid从redis读取该用户权限，判断当前用户是否有这个接口的权限：

- 校验通过: 转发该请求到相应运营后端接口(`${CONFIG.gw_url}`)，并将接口返回的数据返回给前端。

- 校验不通过：中台返回错误码30400

这部分接口配置可查阅配置文件`config/proxy-perm.config.js`

## Oauth2.0登录流程
**Oauth2背景知识**
- 走后端请求: 先拿授权码后拿access_token: 此时的response_type=code;
- 只走前端请求: 直接拿access_token:

#### 后端得到用户信息和权限信息

`
api_url: 网关外网地址;
gw_url: 网关内网地址;
host_url: 前端ajax地址;
ajax_url: 运营后端地址;
gw_url: 网关后端地址
`


- 302 redirect ==> ${api_url}/passport/oauth2/authorize
  ?op=grant_ok
  &redirect_uri=${host_url}/callback
  &response_type=code
  &client_id=${client_id}
  [&auth_code=${ctx.query.code}
  &force_login=2]
  返回code
- passport 302 重定向回${host_url}/callback?auth_code=XXXX 将code[授权码]]拼接在后面
- 中台根据code[授权码] 请求
${gw_url}/auth/oauth2/access_token POST
{
  code,
  client_id: CONFIG.client_id,
  client_secret: CONFIG.client_secret,
  grant_type: 'authorization_code',
  redirect_uri: CONFIG.host_url + 'callback'
}
往/callback地址返回access_token
==> {
  // access_token
  uid, avatar, nickname
}

- getPermission({operator: uid}) => {permissions} 将多层对象降级压平
- 存redis: ['user' + uid]: JSON.stringify({
  ...{uid, avatar, nickname},
  secret: Date.now(),
  permissions,
  permissionsOri
})
- set-Cookie: uid, secret
// 中间层的用户信息和权限信息已拿到

#### 前端校验登录
- checkLogin: !!ctx.session.uid ? next() : goLogin(ctx)
- checkCookies:



## 登录校验/权限校验 (middleware/check)
- checkLogin()
- checkCookies()
- checkPermission()

## 路由说明
/------------------ checkLogin()
  /ajax ----------- checkCookies()    // 登录cookie校验     /router/public
  /ajax/purview---- checkPermission() // 权限校验   /router/ajax
  <!-- =================== -->
  /ompweb----------- No need   内部接口 /router/api
  /nopermission ---- nopermission.html
  /404 ------------- 404.html
  /error ----------- body error-tips
  /logout ---------- 登出
  /callback --------


## 公共接口
- GET /ajax/user/info 用户信息
==> { avatar, nickname, uid }

- /ajax/inner/permission/list  权限信息 [控制到菜单]
==> {devManage, porMange, report, sysManage, workorderManage}

- /ajax/purview/inner/product/list 云产品信息[嵌入ifame]
==> [{module_id, mudule_name, module_ename, url}...]


## 业务接口demo
> sysManage系统管理
- /ajax/purview/inner/user/list 拉取用户列表
- /ajax/purview/inner/grp/list 拉取group列表
- /ajax/purview/inner/user/grp/list?page=1&limit=20&uid=80016420 某个用户下组信息


## 将权限配置和代理配置合在一起配置
> 两者都是基于前端路由
```js
// proxy-permission.config.js
// 去掉空格 replace(/\s+/g, '').split(/[\[\]]/);
// {to 代理地址, perm 权限列表, origin 代理域名, config 请求配置} [get] 为默认
const newStructure = {
  '/kae/statistics/overview  [get]': {
    to: '/kae/console/statistics/overview  [get]',
    perm: ['report.devReport']
  },

  '/kae/statistics/overview  [get]': ['/kae/console/statistics/overview  [get]', ['report.devReport']]

  '/kae/statistics/top  [get]': {
    to: '/kae/console/statistics/top  [get]',
    perm: ['report.devReport']
  }
}

// /router/ajax/index.js
const router = require('koa-router')();
const request = require('../../utils/request');
const { addParams } = require('../../utils/format');
const {gw_url: APIG, /* ajax_url: KOMP */} = require('../../../config/index');

const splitUrl = (url) => url.replace(/\s+/g, '').split(/[\[\]]/);
const mapParams = {
  get: 'query',
  post: 'body'
};

const proxyCtrl = (
  fromMethod, fromUrl, toUrl, toMethod, origin, config
) => router[fromMethod](fromUrl, async(ctx, next) => {
  const params = addParams(ctx, mapParams[fromUrl]);
  await request[toMethod](origin + toUrl, {...params, ...config})
    .then((json) => {
      ctx.body = json;
    })
});


Object.keys(newStructure).forEach((key) => {
  // 支持数组类型配置 和 对象类型配置
  let to, perm, origin, config;
  if(Array.isArray(newStructure[key])) {
    ;[to, perm, origin = APIG, config] = newStructure[key];
  } else {
    ;{to, perm, origin = APIG, config} = newStructure[key];
  }
  const [fromUrl, fromMethod = 'get'] = splitUrl(key);
  const [toUrl, toMethod = 'get'] = splitUrl(to);

  proxyCtrl(fromMethod, fromUrl, toUrl, toMethod, origin, config);
});


```
